/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.base.ui.layout.util;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.base.model.layout.Angle;
import org.fortiss.tooling.base.model.layout.Dimension;
import org.fortiss.tooling.base.model.layout.EOrientation;
import org.fortiss.tooling.base.model.layout.ILayoutData;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.model.layout.LayoutFactory;
import org.fortiss.tooling.base.model.layout.Offset;
import org.fortiss.tooling.base.model.layout.Orientation;
import org.fortiss.tooling.base.model.layout.Point;
import org.fortiss.tooling.base.model.layout.Points;
import org.fortiss.tooling.kernel.base.ui.layout.constants.LayoutKeyConstants;
import org.fortiss.tooling.kernel.base.ui.layout.constants.RecommendedLayoutConstants;

/**
 * Utility methods for {@link ILayoutedModelElement} and {@link ILayoutData}.
 * 
 * @author hoelzl
 * @author wandinger
 * @author hummel
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 9C4FEB62413E5D5B6355B4A7D2360C09
 */
public class LayoutDataUtils {

	/** Returns a {@link Point} layout data object with the given key or null. */
	public static Point getPoint(ILayoutedModelElement lobject, String key) {
		for (final ILayoutData ld : lobject.getLayoutDataList()) {
			if (ld.getKey().equals(key) && ld instanceof Point) {
				return (Point) ld;
			}
		}
		return null;
	}

	/** Sets a {@link Point} layout data. */
	public static void setPoint(ILayoutedModelElement lobject, String key,
			org.eclipse.draw2d.geometry.Point point) {
		Point p = getPoint(lobject, key);
		if (p == null) {
			p = LayoutFactory.eINSTANCE.createPoint();
			p.setKey(key);
			lobject.getLayoutDataList().add(p);
		}
		p.setX(point.x);
		p.setY(point.y);
	}

	/** Returns a {@link Dimension} layout data object with the given key. */
	public static Dimension getDimension(ILayoutedModelElement lobject,
			String key) {
		for (final ILayoutData ld : lobject.getLayoutDataList()) {
			if (ld.getKey().equals(key) && ld instanceof Dimension) {
				return (Dimension) ld;
			}
		}
		return null;
	}

	/** Sets a {@link Dimension} layout data object with the given key. */
	public static void setDimension(ILayoutedModelElement lobject, String key,
			org.eclipse.draw2d.geometry.Dimension size) {
		Dimension d = getDimension(lobject, key);
		if (d == null) {
			d = LayoutFactory.eINSTANCE.createDimension();
			d.setKey(key);
			lobject.getLayoutDataList().add(d);
		}
		d.setWidth(size.width);
		d.setHeight(size.height);
	}

	/** Returns a {@link Orientation} layout data object with the given key. */
	public static Orientation getOrientation(ILayoutedModelElement lobject,
			String key) {
		for (final ILayoutData ld : lobject.getLayoutDataList()) {
			if (ld.getKey().equals(key) || ld instanceof Orientation) {
				return (Orientation) ld;
			}
		}
		return null;
	}

	/** Sets a {@link Orientation} layout data object with the given key. */
	public static void setOrientation(ILayoutedModelElement lobject,
			String key, EOrientation orientation) {
		Orientation o = getOrientation(lobject, key);
		if (o == null) {
			o = LayoutFactory.eINSTANCE.createOrientation();
			o.setKey(key);
			lobject.getLayoutDataList().add(o);
		}
		o.setOrientation(orientation);
	}

	/** Returns a {@link Offset} layout data object with the given key. */
	public static Offset getOffset(ILayoutedModelElement lobject, String key) {
		for (final ILayoutData ld : lobject.getLayoutDataList()) {
			if (ld.getKey().equals(key) || ld instanceof Offset) {
				return (Offset) ld;
			}
		}
		return null;
	}

	/** Sets a {@link Offset} layout data object with the given key. */
	public static void setOffset(ILayoutedModelElement lobject, String key,
			int offset) {
		Offset o = getOffset(lobject, key);
		if (o == null) {
			o = LayoutFactory.eINSTANCE.createOffset();
			o.setKey(key);
			lobject.getLayoutDataList().add(o);
		}
		o.setOffset(offset);
	}

	/** Returns an angle. */
	public static Angle getAngle(ILayoutedModelElement lobject, String key) {
		for (final ILayoutData ld : lobject.getLayoutDataList()) {
			if (ld.getKey().equals(key) && ld instanceof Angle) {
				return (Angle) ld;
			}
		}
		return null;
	}

	/** Returns the angle as double value (0 if not existing). */
	public static double getAngleAsDouble(ILayoutedModelElement lobject,
			String key) {
		final Angle a = getAngle(lobject, key);
		if (a == null) {
			return 0;
		}
		return a.getAngle();
	}

	/** Sets a {@link Point} layout data. */
	public static void setAngle(ILayoutedModelElement lobject, String key,
			double angle) {
		Angle a = getAngle(lobject, key);
		if (a == null) {
			a = LayoutFactory.eINSTANCE.createAngle();
			a.setKey(key);
			lobject.getLayoutDataList().add(a);
		}
		a.setAngle(angle);
	}

	/** Gets the position stored at the given key. */
	public static org.eclipse.draw2d.geometry.Point getPosition(
			ILayoutedModelElement layouted, String key) {
		final Point point = LayoutDataUtils.getPoint(layouted, key);

		if (point == null) {
			int index = 0;
			final EObject container = layouted.eContainer();
			if (container != null) {
				index = 1 + container.eContents().indexOf(layouted);
			}

			return new org.eclipse.draw2d.geometry.Point(index * 20, index * 20);
		}

		return new org.eclipse.draw2d.geometry.Point(point.getX(), point.getY());
	}

	/** Returns a {@link Points} layout data object with the given key. */
	public static Points getPoints(ILayoutedModelElement lobject, String key) {
		for (final ILayoutData ld : lobject.getLayoutDataList()) {
			if (ld.getKey().equals(key) || ld instanceof Points) {
				return (Points) ld;
			}
		}
		return null;
	}

	/**
	 * Returns the bounds of the given layouted node.
	 */
	public static Rectangle getNodeBounds(ILayoutedModelElement layouted) {
		return new Rectangle(getNodePosition(layouted), getNodeSize(layouted));
	}

	/**
	 * Sets the bounds of the given layouted node.
	 */
	public static void setNodeBounds(ILayoutedModelElement layouted,
			Rectangle bounds) {
		setNodePosition(layouted, bounds.getLocation());
		setNodeSize(layouted, bounds.getSize());
	}

	/** Returns the position of the given layouted node. */
	public static org.eclipse.draw2d.geometry.Point getNodePosition(
			ILayoutedModelElement layouted) {
		return getPosition(layouted, LayoutKeyConstants.NODE_POSITION);
	}

	/**
	 * Sets the layouted node's position.
	 */
	public static void setNodePosition(ILayoutedModelElement layouted,
			org.eclipse.draw2d.geometry.Point position) {
		setPoint(layouted, LayoutKeyConstants.NODE_POSITION, position);
	}

	/** Returns the size of the given layouted node. */
	public static org.eclipse.draw2d.geometry.Dimension getNodeSize(
			ILayoutedModelElement layouted) {
		final Dimension size = LayoutDataUtils.getDimension(layouted,
				LayoutKeyConstants.NODE_DIMENSION);

		if (size == null) {
			return new org.eclipse.draw2d.geometry.Dimension(
					RecommendedLayoutConstants.RECTANGULAR_SHAPE_MINIMUM_WIDTH,
					RecommendedLayoutConstants.RECTANGULAR_SHAPE_MINIMUM_HEIGHT);
		}
		return new org.eclipse.draw2d.geometry.Dimension(size.getWidth(),
				size.getHeight());
	}

	/** Sets the layouted node's size. */
	public static void setNodeSize(ILayoutedModelElement layouted,
			org.eclipse.draw2d.geometry.Dimension size) {
		setDimension(layouted, LayoutKeyConstants.NODE_DIMENSION, size);
	}

	/**
	 * Returns the layouted connector's position.
	 */
	public static org.eclipse.draw2d.geometry.Point getConnectorPosition(
			ILayoutedModelElement layouted) {
		return getPosition(layouted, LayoutKeyConstants.CONNECTOR_POSITION);
	}

	/**
	 * Sets the layouted connector's position.
	 */
	public static void setConnectorPosition(ILayoutedModelElement layouted,
			org.eclipse.draw2d.geometry.Point position) {
		setPoint(layouted, LayoutKeyConstants.CONNECTOR_POSITION, position);
	}

	/**
	 * Returns the layouted connector's orientation.
	 */
	public static EOrientation getConnectorOrientation(
			ILayoutedModelElement layouted) {
		final Orientation orientation = getOrientation(layouted,
				LayoutKeyConstants.CONNECTOR_ORIENTATION);
		if (orientation == null) {
			return EOrientation.NORTH;
		}
		return orientation.getOrientation();
	}

	/**
	 * Sets the layouted connector's orientation.
	 */
	public static void setConnectorOrientation(ILayoutedModelElement layouted,
			EOrientation orientation) {
		setOrientation(layouted, LayoutKeyConstants.CONNECTOR_ORIENTATION,
				orientation);
	}

	/**
	 * Returns the layouted connector's offset.
	 */
	public static int getConnectorOffset(ILayoutedModelElement layouted) {
		final Offset offset = getOffset(layouted,
				LayoutKeyConstants.CONNECTOR_OFFSET);
		if (offset == null) {
			return 0;
		}
		return offset.getOffset();
	}

	/**
	 * Sets the layouted connector's angle.
	 */
	public static void setConnectorOffset(ILayoutedModelElement layouted,
			int offset) {
		setOffset(layouted, LayoutKeyConstants.CONNECTOR_OFFSET, offset);
	}

	/**
	 * Sets the layouted connector's offset.
	 */
	public static void setConnectorAngle(ILayoutedModelElement layouted,
			double angle) {
		setAngle(layouted, LayoutKeyConstants.CONNECTOR_OFFSET, angle);
	}

	/**
	 * Returns the layouted connector's angle.
	 */
	public static double getConnectorAngle(ILayoutedModelElement layouted) {
		final Angle angle = getAngle(layouted,
				LayoutKeyConstants.CONNECTOR_ANGLE);
		if (angle == null) {
			return 0;
		}
		return angle.getAngle();
	}

	/**
	 * Creates a new connection layout.
	 */
	public static void createConnectionLayout(ILayoutedModelElement lo) {
		final Points points = LayoutFactory.eINSTANCE.createPoints();
		points.setKey(LayoutKeyConstants.CONNECTION_POINTS);
		lo.getLayoutDataList().add(points);
	}

	/**
	 * Creates a new connector layout.
	 */
	public static void createConnectorLayout(ILayoutedModelElement lo) {
		createNodeLayout(lo);

		final Point cpos = LayoutFactory.eINSTANCE.createPoint();
		cpos.setKey(LayoutKeyConstants.CONNECTOR_POSITION);

		final Dimension cdim = LayoutFactory.eINSTANCE.createDimension();
		cdim.setKey(LayoutKeyConstants.CONNECTOR_DIMENSION);

		lo.getLayoutDataList().add(cpos);
		lo.getLayoutDataList().add(cdim);
	}

	/**
	 * Creates a new node layout.
	 */
	public static void createNodeLayout(ILayoutedModelElement lo) {
		final Point pos = LayoutFactory.eINSTANCE.createPoint();
		pos.setKey(LayoutKeyConstants.NODE_POSITION);

		final Dimension dim = LayoutFactory.eINSTANCE.createDimension();
		dim.setKey(LayoutKeyConstants.NODE_DIMENSION);

		lo.getLayoutDataList().add(pos);
		lo.getLayoutDataList().add(dim);
	}

}
