/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal.storage.eclipse;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.IdentityHashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.interfaces.IStorageProvider;
import org.fortiss.tooling.kernel.interfaces.ITopLevelElementContext;
import org.fortiss.tooling.kernel.services.IPersistencyService;
import org.fortiss.tooling.kernel.util.LoggingUtils;

/**
 * This class implements the persistency service behavior for Eclipse file
 * storage.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 350FE8BC692DF5B142EE95493B393B3B
 */
public class EclipseResourceStorageProvider implements IResourceChangeListener,
		IResourceDeltaVisitor, IStorageProvider {
	/** Cache of models loaded so far. */
	private final Map<IFile, ModelContext> loadedContexts = new HashMap<IFile, ModelContext>();

	/** Map from toplevel elements to context. */
	private final Map<EObject, ModelContext> rootElementContexts = new IdentityHashMap<EObject, ModelContext>();

	/** Constructor. */
	public EclipseResourceStorageProvider() {
		ResourcesPlugin.getWorkspace().addResourceChangeListener(this);
		searchWorkspaceForModels();
	}

	/** Searches existing models in Eclipse project roots. */
	private synchronized void searchWorkspaceForModels() {
		for (IProject project : ResourcesPlugin.getWorkspace().getRoot()
				.getProjects()) {
			if (!project.isOpen()) {
				continue;
			}
			try {
				for (IResource res : project.members()) {
					if (res instanceof IFile) {
						IFile file = (IFile) res;
						// TODO (FH): use an extension mechanism for this check
						if (file.getFileExtension().equals("af3_20")) {
							loadContext(file);
						}
					}
				}
			} catch (CoreException e) {
				// ignore
			} catch (IOException ioex) {
				// ignore
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public void resourceChanged(IResourceChangeEvent event) {
		synchronized (this) {
			if (event.getDelta() == null) {
				return;
			}
			try {
				event.getDelta().accept(this);
			} catch (final CoreException e) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						"Should not happen!", e);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public boolean visit(IResourceDelta delta) {
		if (delta.getResource() instanceof IFile
				&& delta.getResource().getParent() instanceof IProject
				&& delta.getResource().getFileExtension().equals("af3_20")) {
			IFile file = (IFile) delta.getResource();
			if (delta.getKind() == IResourceDelta.ADDED) {
				try {
					loadContext(file);
				} catch (IOException ioex) {
					LoggingUtils.error(ToolingKernelActivator.getDefault(),
							"Had an error during reloading the file!", ioex);
				}
			} else if (delta.getKind() == IResourceDelta.REMOVED
					&& isLoaded(file)) {
				unloadContext((IFile) delta.getResource());
			} else if (delta.getKind() == IResourceDelta.CHANGED) {
				handleChange((IFile) delta.getResource());
			}

			// Refresh the top-level elements
			IPersistencyService.INSTANCE
					.refreshTopLevelElements(EclipseResourceStorageProvider.this);
		}
		return true;
	}

	/** Handles a change of the given file (which is actually managed). */
	private void handleChange(final IFile file) {
		final ModelContext context = loadedContexts.get(file);
		if (!context.getLastChangeWasIntended()) {
			// TODO (FH): find a solution for this ui code
			// Display.getDefault().asyncExec(new Runnable() {
			// @Override
			// public void run() {
			// final boolean reload = true;
			// // MessageUtils.askQuestion(
			// // "Reload changed file?",
			// // "The file "
			// // + file.getName()
			// // + " changed on disk. "
			// // + "Load these changes? "
			// // +
			// // "Note that loading the changes will discard all editors, "
			// // +
			// // "so all unsaved changes and the undo history will be lost. "
			// // + "However if you do not reload now, "
			// // + "the contents of the file will be overwritten "
			// // + "the next time you perform a save.");
			//
			// if (reload) {
			// }
			//
			// }
			// });
			unloadContext(file);
			try {
				loadContext(file).setLastChangeWasIntended();

				// perform touch to make sure the viewer does a
				// refresh
				file.touch(null);
			} catch (final Exception e) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						"Had an error during reloading the file!", e);
			}
		}
	}

	/** Returns whether the given file has been loaded by this manager. */
	public synchronized boolean isLoaded(IFile file) {
		return loadedContexts.containsKey(file);
	}

	/** Loads a file and creates a new context from its contents. */
	private ModelContext loadContext(IFile file) throws IOException {
		ModelContext mc = new ModelContext(file);
		loadedContexts.put(file, mc);
		rootElementContexts.put(mc.getTopLevelElement(), mc);
		return mc;
	}

	/** Loads a file and creates a new context from its contents. */
	private void unloadContext(IFile file) {
		final ModelContext context = loadedContexts.remove(file);
		if (context != null) {
			rootElementContexts.remove(context.getTopLevelElement());
			context.destroy();
		}
	}

	/** {@inheritDoc} */
	@Override
	public List<ITopLevelElementContext> getTopLevelElementContexts() {
		List<ITopLevelElementContext> result = new ArrayList<ITopLevelElementContext>();
		result.addAll(rootElementContexts.values());
		return result;
	}
}
